from django.template import Context, Template
from django.template.loader import render_to_string
from django.utils.encoding import force_unicode

"""
IMPORTANT:
Sidebar Widget itself is not a django form widget, but a term of small block/pieces of
information that lives on various part of web page, could be display some
information or have specific functionality. Easily configured and re-ordered
with other widgets.

Yes, its maybe little bit confusing between Django form widget and sidebar widget.
but i have no other words to name or describes it :)

2012 - Eka Putra
"""

class SidebarWidget(object):
    """
    Base class for sidebar widget.
    Each sidebar widget must be subclass of this class.
    """
    admin_form = None
    template_text = ''
    template_file = ''

    def __init__(self, name, description):
        self.id = self.__class__.__name__
        self.name = force_unicode(name)
        self.description = force_unicode(description)

    def render(self, title, params=None):
        """
        Widget render method, rendering method based on template type
        template_text --> template that specified in the code itself.
        template_file --> template is a file.
        """
        if params:
            params.update({'widget_title': title})
        else:
            params = {}
            params['widget_title'] = title
            
        output = ''
        # We need to anticipate if both template_text and template_file defined
        # So, return result of template_text as priority
        if self.template_text:
            template = Template(self.template_text)
            try:
                output = template.render(Context(params))
            except:
                pass
            else:
                return output

        if self.template_file:
            try:
                output = render_to_string(self.template_file, params)
            except:
                pass
            else:
                return output

        return output

sidebar_widget = None
"""
@variable : sidebar_widget

this variable must be used to declaring a new sidebar widget subclass.
without it, new sidebar widget will not be visible in Sidebar Admin page.
"""




from sidebar.base import SidebarWidget, sidebar_widget
from django import forms

"""
TEMPLATE : Can be a template file or just a short text like below.
all variable came from a admin_form value, any form field declared in
admin_form Form, will be available in template context.

Available context variable in template is:
    1. All form field by admin_form Form
    2. A widget title {{widget_title}}

An admin form is optional, but TEMPLATE must exists.
"""

TEMPLATE = """
<div class="side_block">
{{text|safe}}
</div>
"""

class TextForm(forms.Form):
    """ Just a normal django form """
    text = forms.CharField(widget=forms.Textarea)

class TextSidebarWidget(SidebarWidget):
    """ Sidebar class must a subclass of SidebarWidget class"""
    
    admin_form = TextForm
    template_text = TEMPLATE

# Here we register out Widget, New widget instance must be in sidebar_widget variable
sidebar_widget = TextSidebarWidget(
                            'Text Widget',
                            'Display custom text or Html'
                            )




from django.utils.importlib import import_module
from django.utils import simplejson
from django.http import HttpResponse
from django.conf import settings

from sidebar.models import Sidebar, SidebarWidget
from sidebar.forms import WidgetTitleForm, DefaultAdminForm

def form_to_json(form):
    """
    Convert form data (field name and value) to json format,
    in this format we store the data in the database.
    """

    form_data = {}
    for field in form:

        #if form data is an object (eg. ModelChoiceField), we take different
        #way to save
        if isinstance(form.cleaned_data[field.name], object) and getattr(form.cleaned_data[field.name],'id',''):
            obj = form.cleaned_data[field.name]
            form_data[field.name] = obj.id
        else:
            form_data[field.name] = form.cleaned_data[field.name]
    return simplejson.dumps(form_data)

def json_to_dict(json_data):
    """Convert json formated data to its original format"""
    return simplejson.loads(json_data)

def get_sidebar_widgets():
    """Get all available widgets."""

    widgets = []
    for widget_id in settings.AVAILABLE_SIDEBAR_WIDGETS:
        try:
            wgt = import_module('sidebar_widgets.%s' % widget_id)
            reload(wgt)
        except:
            pass
        else:
            widgets.append(wgt.sidebar_widget)
    return widgets


def get_widget_name(widget_id):
    """Get widget name from widget ID"""

    name = ''

    widgets = get_sidebar_widgets()
    for widget in widgets:
        if widget.id == widget_id:
            name = widget.name
            break

    return name


def add_widget(widget_id, sidebar_id):
    """Add widget to sidebar"""

    sidebar = None
    try:
        sidebar = Sidebar.objects.get(pk=sidebar_id)
    except Sidebar.DoesNotExist:
        return
    else:
        name = get_widget_name(widget_id)
        widget = SidebarWidget()
        widget.title = name
        widget.sidebar = sidebar
        widget.widget = widget_id
        widget.save()
        return True


def render_single_widget(widgets, widget_item):
    """Render a single widget and return the output"""

    widget = None

    for w in widgets:
        if w.id == widget_item.widget:
            widget = w
            break
    # If widget found, render it
    if widget:
        widget_data = widget_item.content
        try:
            widget_params = json_to_dict(widget_data)
        except:
            output = widget.render(widget_item.title, None)
        else:
            output = widget.render(widget_item.title, widget_params)
        return output

    return False


def render_widgets(widget_list):
    """
    Loop through widget_list and render each of them.
    and return all result at once.
    """

    widgets = get_sidebar_widgets()
    output = []

    for widget in widget_list:
        single_widget = render_single_widget(widgets, widget)
        if single_widget:
            output.append(single_widget)

    return u''.join(output)


def get_full_widget_data(widgets, widget_item):
    """Get full widget data"""

    widget = None
    for w in widgets:
        if w.id == widget_item.widget:
            widget = w
            break

    if widget:
        data = {}
        data['id'] = widget_item.id
        data['title'] = widget_item.title
        data['type'] = widget_item.widget

        data['title_form'] = WidgetTitleForm(
                                            initial={'widget_title': data['title']}
                                            )
        if widget.admin_form:
            try:
                initial=json_to_dict(widget_item.content)
            except:
                data['admin_form'] = widget.admin_form()
            else:
                data['admin_form'] = widget.admin_form(
                                            initial=initial
                                            )
        return data

    return False


def get_used_widgets(sidebar_id):
    """get full widgets data for current sidebar, to show in admin page."""

    widgets = get_sidebar_widgets()
    sidebar = None
    all_used_widget = []

    try:
        sidebar = Sidebar.objects.get(pk=sidebar_id)
    except Sidebar.DoesNotExist:
        pass
    else:
        widget_list = sidebar.sidebarwidget_set.all()
        widget_list = list(widget_list)

        for widget in widget_list:
            used_widget_data = get_full_widget_data(widgets, widget)
            if used_widget_data:
                all_used_widget.append(used_widget_data)

    return all_used_widget


def update_widget(request):
    """ Update single Widget """

    result = {}
    widgets = get_sidebar_widgets()
    widget_id = request.POST.get('widget_id', 0)

    try:
        widget_item = SidebarWidget.objects.get(pk=widget_id)
    except SidebarWidget.DoesNotExist:
        return result
    else:
        widget = None
        for w in widgets:
            if w.id == widget_item.widget:
                widget = w
                break
        if not widget:
            return result
        else:
            title_form = WidgetTitleForm(request.POST)
            #prepare widget admin form for validation
            if widget.admin_form:
                admin_form = widget.admin_form(request.POST)
            else:
                admin_form = DefaultAdminForm(request.POST)

            if title_form.is_valid() and admin_form.is_valid():
                widget_item.title = title_form.cleaned_data['widget_title']
                widget_item.content = form_to_json(admin_form)
                widget_item.save()
                result['status'] = 'success'
                result['title'] = title_form.cleaned_data['widget_title']
            else:
                error_fields = {}
                error_fields.update(title_form.errors)
                error_fields.update(admin_form.errors)
                result['error_fields'] = [field for field in error_fields]

    return result

def reorder_widgets(widgets):
    """Reorder widget positions"""

    widget_ids = widgets.split(',')
    count = 1
    #main loop to re-set menu order value
    for w_id in widget_ids:
        try:
            widget = SidebarWidget.objects.get(pk=w_id)
        except:
            pass
        else:
            widget.order = count
            widget.save()
            count += 1
    return

def delete_widget(widget_id):
    """Delete widget from"""

    try:
        widget = SidebarWidget.objects.get(pk=widget_id)
    except:
        return
    else:
        widget.delete()
        return




from django.contrib import admin
from sidebar.models import Sidebar
from django import forms
from django.db import models

class SidebarAdmin(admin.ModelAdmin):
    fields = ('position',)

admin.site.register(Sidebar, SidebarAdmin)




from django.http import HttpResponse
from django.shortcuts import render_to_response
from django.utils import simplejson
from django.template import RequestContext
from django.contrib.auth.decorators import login_required

from sidebar.utils import add_widget, update_widget, reorder_widgets
from sidebar.utils import delete_widget, get_used_widgets

@login_required
def ajax_load_widgets(request):
    """load sidebar ajax"""

    if request.is_ajax():
        if request.method == 'POST':
            sidebar_id = request.POST.get('sidebar_id', 0)
            widgets = get_used_widgets(sidebar_id)

            return render_to_response(
                                'admin/sidebar/sidebar/widgets.html',
                                {'used_widgets': widgets},
                                context_instance = RequestContext(request)
                                )

@login_required
def ajax_add_to_sidebar(request):
    """ Add widget to sidebar """

    if request.is_ajax():
        if request.method == 'POST':
            widget = request.POST.get('widget','')
            sidebar = request.POST.get('sidebar', 0)
            result = add_widget(widget, int(sidebar))
            if result:
                return HttpResponse(
                                simplejson.dumps({'status':'1'}),
                                content_type='application/javascript; charset=utf-8;')
            else:
                return HttpResponse(
                                simplejson.dumps({'status':'0'}),
                                content_type='application/javascript; charset=utf-8;')
                                
@login_required
def ajax_update_widget(request):
    """ Update single widget """

    if request.is_ajax():
        if request.method == 'POST':
            result = update_widget(request)
            return HttpResponse(
                            simplejson.dumps(result),
                            content_type='application/javascript; charset=utf-8;')
                            
@login_required
def ajax_reorder_widget(request):
    """ Reorder widget position """

    if request.is_ajax():
        if request.method == 'POST':
            widgets = request.POST.get('order','')
            if widgets:
                reorder_widgets(widgets)
            return HttpResponse(simplejson.dumps({'status':'success'}),
                                content_type='application/javascript; charset=utf-8;')
                                
@login_required
def ajax_delete_widget(request):
    """ Delete single widget """

    if request.is_ajax():
        if request.method == 'POST':
            widget_id = request.POST.get('widget_id', 0)
            if widget_id:
                delete_widget(widget_id)
            return HttpResponse(simplejson.dumps({'status':'success'}),
                                content_type='application/javascript; charset=utf-8;')




try: 
    from django.conf.urls import patterns, url 
except ImportError: 
    # for Django version less then 1.4
    from django.conf.urls.defaults import patterns, url


urlpatterns = patterns('sidebar.views',
    url(r'^sidebar/$', 'ajax_load_widgets', name='load_sidebar'),
    url(r'^sidebar/add/$', 'ajax_add_to_sidebar', name='sidebar_add'),
    url(r'^sidebar/update/$', 'ajax_update_widget', name='update_widget'),
    url(r'^sidebar/reorder/$', 'ajax_reorder_widget', name='reorder_widget'),
    url(r'^sidebar/delete/$', 'ajax_delete_widget', name='delete_widget'),
)




from django import forms
from django.utils.translation import ugettext_lazy as _

class WidgetTitleForm(forms.Form):
    """A simple form for widget title"""
    
    widget_title = forms.CharField(label=_('Title'), max_length=50)

class DefaultAdminForm(forms.Form):
    """
    A default admin form, in case widget doesn't have admin form.
    because admin form is required (to simplify validation process) in widget update. 
    this form will be used, and validated and always be valid, since we do
    nothing here.
    """
    pass



from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.conf import settings

class Sidebar(models.Model):
    position = models.CharField(_('Sidebar Position'),
                                max_length=100,
                                unique=True,
                                choices=settings.AVAILABLE_SIDEBARS,
                                help_text=_('This is position of the sidebar.'))

    def __unicode__(self):
        for position in settings.AVAILABLE_SIDEBARS:
            if position[0] == self.position:
                return u'%s' % position[1]


    class Meta:
        verbose_name = _('Sidebar')
        verbose_name_plural = _('Sidebars')

class SidebarWidget(models.Model):
    title = models.CharField(max_length=150)
    sidebar = models.ForeignKey(Sidebar)
    widget = models.CharField(max_length=50)
    content = models.TextField(blank=True, null=True)
    order = models.IntegerField(blank=True, null=True, default=100)

    def __unicode__(self):
        return self.title

    class Meta:
        ordering = ('order',)
        db_table = 'sidebar_widget'




from django import template
register = template.Library()

from sidebar.utils import get_sidebar_widgets, render_widgets, get_used_widgets
from sidebar.models import Sidebar, SidebarWidget

@register.inclusion_tag('admin/sidebar/sidebar/sidebar_builder.html')
def sidebar_builder(object_id):
    """
    Sidebar Builder tags to use in django admin Sidebar manager.
    """

    data = {
        'sidebar_id': object_id,
        'available_widgets' : get_sidebar_widgets(),
        }

    return data

@register.simple_tag
def get_sidebar(position_id):
    """
    Tags to pull sidebar content(widgets) into the location where this tag
    was called.
    """
    sidebar = None
    try:
        sidebar = Sidebar.objects.get(position=position_id)
    finally:

        if sidebar:
            widgets = sidebar.sidebarwidget_set.all()
            widgets = list(widgets)
            return render_widgets(widgets)

        else:
            return ''

